@@ -351,7 +351,7 @@ func runAnalysis(ctx context.Context, snapshot *snapshot, analyzer *analysis.Ana
 	}
 
 	for _, diag := range diagnostics {
-		srcDiags, err := analysisDiagnosticDiagnostics(ctx, snapshot, pkg, analyzer, diag)
+		srcDiags, err := analysisDiagnosticDiagnostics(snapshot, pkg, analyzer, diag)
 		if err != nil {
 			event.Error(ctx, "unable to compute analysis error position", err, tag.Category.Of(diag.Category), tag.Package.Of(pkg.ID()))
 			continue
@@ -423,6 +423,7 @@ func (s *snapshot) DiagnosePackage(ctx context.Context, spkg source.Package) (ma
 				clone := *diag
 				clone.SuggestedFixes = eaDiag.SuggestedFixes
 				clone.Tags = eaDiag.Tags
+				clone.Analyzer = eaDiag.Analyzer
 				diag = &clone
 			}
 		}
